@@ -10,35 +10,55 @@
 /*	into the \fBincoming\fR mail queue, and informs the queue
 /*	manager of its arrival.
 /*
-/*	The \fBcleanup\fR(8) daemon always performs the following transformations:
+/*	The \fBcleanup\fR(8) daemon performs the following transformations:
 /* .IP \(bu
 /*	Insert missing message headers: (\fBResent-\fR) \fBFrom:\fR,
 /*	\fBTo:\fR, \fBMessage-Id:\fR, and \fBDate:\fR.
+/* .br
+/*	This is enabled with the \fBlocal_header_rewrite_clients\fR and
+/*	\fBalways_add_missing_headers\fR parameter settings.
 /* .IP \(bu
 /*	Transform envelope and header addresses to the standard
 /*	\fIuser@fully-qualified-domain\fR form that is expected by other
 /*	Postfix programs.
-/*	This task is delegated to the \fBtrivial-rewrite\fR(8) daemon.
+/*	This task depends on the \fBtrivial-rewrite\fR(8) daemon.
+/* .br
+/*	The header transformation is enabled with the
+/*	\fBlocal_header_rewrite_clients\fR parameter setting.
 /* .IP \(bu
 /*	Eliminate duplicate envelope recipient addresses.
-/* .PP
-/*	The following address transformations are optional:
+/* .br
+/*	This is enabled with the \fBduplicate_filter_limit\fR
+/*	parameter setting.
+/* .IP \(bu
+/*	Remove message headers: \fBBcc\fR, \fBContent-Length\fR,
+/*	\fBResent-Bcc\fR, \fBReturn-Path\fR.
+/* .br
+/*	This is enabled with the message_drop_headers parameter
+/*	setting.
 /* .IP \(bu
 /*	Optionally, rewrite all envelope and header addresses according
 /*	to the mappings specified in the \fBcanonical\fR(5) lookup tables.
+/* .br
+/*	The header transformation is enabled with the
+/*	\fBlocal_header_rewrite_clients\fR parameter setting.
 /* .IP \(bu
 /*	Optionally, masquerade envelope sender addresses and message
 /*	header addresses (i.e. strip host or domain information below
 /*	all domains listed in the \fBmasquerade_domains\fR parameter,
 /*	except for user names listed in \fBmasquerade_exceptions\fR).
 /*	By default, address masquerading does not affect envelope recipients.
+/* .br
+/*	The header transformation is enabled with the
+/*	\fBlocal_header_rewrite_clients\fR parameter setting.
 /* .IP \(bu
 /*	Optionally, expand envelope recipients according to information
-/*	found in the \fBvirtual\fR(5) lookup tables.
+/*	found in the \fBvirtual_alias_maps\fR lookup tables.
 /* .PP
 /*	The \fBcleanup\fR(8) daemon performs sanity checks on the content of
 /*	each message. When it finds a problem, by default it returns a
-/*	diagnostic status to the client, and leaves it up to the client
+/*	diagnostic status to the cleanup service client, and leaves
+/*	it up to the client
 /*	to deal with the problem. Alternatively, the client can request
 /*	the \fBcleanup\fR(8) daemon to bounce the message back to the sender
 /*	in case of trouble.
@@ -46,10 +66,13 @@
 /*	RFC 822 (ARPA Internet Text Messages)
 /*	RFC 2045 (MIME: Format of Internet Message Bodies)
 /*	RFC 2046 (MIME: Media Types)
+/*	RFC 2822 (Internet Message Format)
 /*	RFC 3463 (Enhanced Status Codes)
 /*	RFC 3464 (Delivery status notifications)
+/*	RFC 5322 (Internet Message Format)
 /* DIAGNOSTICS
-/*	Problems and transactions are logged to \fBsyslogd\fR(8).
+/*	Problems and transactions are logged to \fBsyslogd\fR(8)
+/*	or \fBpostlogd\fR(8).
 /* BUGS
 /*	Table-driven rewriting rules make it hard to express \fBif then
 /*	else\fR and other logical relationships.
@@ -66,7 +89,7 @@
 /* COMPATIBILITY CONTROLS
 /* .ad
 /* .fi
-/* .IP "\fBundisclosed_recipients_header (To: undisclosed-recipients:;)\fR"
+/* .IP "\fBundisclosed_recipients_header (see 'postconf -d' output)\fR"
 /*	Message header that the Postfix \fBcleanup\fR(8) server inserts when a
 /*	message contains no To: or Cc: message header.
 /* .PP
@@ -74,9 +97,35 @@
 /* .IP "\fBenable_errors_to (no)\fR"
 /*	Report mail delivery errors to the address specified with the
 /*	non-standard Errors-To: message header, instead of the envelope
-/*	sender address (this feature is removed with Postfix 2.2, is
-/*	turned off by default with Postfix 2.1, and is always turned on
+/*	sender address (this feature is removed with Postfix version 2.2, is
+/*	turned off by default with Postfix version 2.1, and is always turned on
 /*	with older Postfix versions).
+/* .PP
+/*	Available in Postfix version 2.6 and later:
+/* .IP "\fBalways_add_missing_headers (no)\fR"
+/*	Always add (Resent-) From:, To:, Date: or Message-ID: headers
+/*	when not present.
+/* .PP
+/*	Available in Postfix version 2.9 and later:
+/* .IP "\fBenable_long_queue_ids (no)\fR"
+/*	Enable long, non-repeating, queue IDs (queue file names).
+/* HEADER FORMATTING CONTROLS
+/* .ad
+/* .fi
+/* .PP
+/*	Available in Postfix version 3.0 and later:
+/* .IP "\fBmessage_drop_headers (bcc, content-length, resent-bcc, return-path)\fR"
+/*	Names of message headers that the \fBcleanup\fR(8) daemon will remove
+/*	after applying \fBheader_checks\fR(5) and before invoking Milter applications.
+/* .PP
+/*	Available in Postfix version 3.3 and later:
+/* .IP "\fBheader_from_format (standard)\fR"
+/*	The format of the Postfix-generated \fBFrom:\fR header.
+/* .PP
+/*	Available in Postfix version 3.10 and later:
+/* .IP "\fBfull_name_encoding_charset (utf-8)\fR"
+/*	The character set name (also called "charset") that Postfix
+/*	will output when it generates an RFC 2047 encoded full name.
 /* BUILT-IN CONTENT FILTERING CONTROLS
 /* .ad
 /* .fi
@@ -108,6 +157,83 @@
 /* .IP "\fBmessage_strip_characters (empty)\fR"
 /*	The set of characters that Postfix will remove from message
 /*	content.
+/* .PP
+/*	Available in Postfix version 3.9, 3.8.5, 3.7.10, 3.6.14,
+/*	3.5.24, and later:
+/* .IP "\fBcleanup_replace_stray_cr_lf (yes)\fR"
+/*	Replace each stray <CR> or <LF> character in message
+/*	content with a space character, to prevent outbound SMTP smuggling,
+/*	and to make the evaluation of Postfix-added DKIM or other signatures
+/*	independent from how a remote mail server handles such characters.
+/* BEFORE QUEUE MILTER CONTROLS
+/* .ad
+/* .fi
+/*	As of version 2.3, Postfix supports the Sendmail version 8
+/*	Milter (mail filter) protocol. When mail is not received via
+/*	the smtpd(8) server, the cleanup(8) server will simulate
+/*	SMTP events to the extent that this is possible. For details
+/*	see the MILTER_README document.
+/* .IP "\fBnon_smtpd_milters (empty)\fR"
+/*	A list of Milter (mail filter) applications for new mail that
+/*	does not arrive via the Postfix \fBsmtpd\fR(8) server.
+/* .IP "\fBmilter_protocol (6)\fR"
+/*	The mail filter protocol version and optional protocol extensions
+/*	for communication with a Milter application; prior to Postfix 2.6
+/*	the default protocol is 2.
+/* .IP "\fBmilter_default_action (tempfail)\fR"
+/*	The default action when a Milter (mail filter) response is
+/*	unavailable (for example, bad Postfix configuration or Milter
+/*	failure).
+/* .IP "\fBmilter_macro_daemon_name ($myhostname)\fR"
+/*	The {daemon_name} macro value for Milter (mail filter) applications.
+/* .IP "\fBmilter_macro_v ($mail_name $mail_version)\fR"
+/*	The {v} macro value for Milter (mail filter) applications.
+/* .IP "\fBmilter_connect_timeout (30s)\fR"
+/*	The time limit for connecting to a Milter (mail filter)
+/*	application, and for negotiating protocol options.
+/* .IP "\fBmilter_command_timeout (30s)\fR"
+/*	The time limit for sending an SMTP command to a Milter (mail
+/*	filter) application, and for receiving the response.
+/* .IP "\fBmilter_content_timeout (300s)\fR"
+/*	The time limit for sending message content to a Milter (mail
+/*	filter) application, and for receiving the response.
+/* .IP "\fBmilter_connect_macros (see 'postconf -d' output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after completion of an SMTP connection.
+/* .IP "\fBmilter_helo_macros (see 'postconf -d' output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the SMTP HELO or EHLO command.
+/* .IP "\fBmilter_mail_macros (see 'postconf -d' output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the SMTP MAIL FROM command.
+/* .IP "\fBmilter_rcpt_macros (see 'postconf -d' output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the SMTP RCPT TO command.
+/* .IP "\fBmilter_data_macros (see 'postconf -d' output)\fR"
+/*	The macros that are sent to version 4 or higher Milter (mail
+/*	filter) applications after the SMTP DATA command.
+/* .IP "\fBmilter_unknown_command_macros (see 'postconf -d' output)\fR"
+/*	The macros that are sent to version 3 or higher Milter (mail
+/*	filter) applications after an unknown SMTP command.
+/* .IP "\fBmilter_end_of_data_macros (see 'postconf -d' output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the message end-of-data.
+/* .PP
+/*	Available in Postfix version 2.5 and later:
+/* .IP "\fBmilter_end_of_header_macros (see 'postconf -d' output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the end of the message header.
+/* .PP
+/*	Available in Postfix version 2.7 and later:
+/* .IP "\fBmilter_header_checks (empty)\fR"
+/*	Optional lookup tables for content inspection of message headers
+/*	that are produced by Milter applications.
+/* .PP
+/*	Available in Postfix version 3.1 and later:
+/* .IP "\fBmilter_macro_defaults (empty)\fR"
+/*	Optional list of \fIname=value\fR pairs that specify default
+/*	values for arbitrary macros that Postfix may send to Milter
+/*	applications.
 /* MIME PROCESSING CONTROLS
 /* .ad
 /* .fi
@@ -128,6 +254,12 @@
 /* .IP "\fBstrict_mime_encoding_domain (no)\fR"
 /*	Reject mail with invalid Content-Transfer-Encoding: information
 /*	for the message/* or multipart/* MIME content types.
+/* .PP
+/*	Available in Postfix version 2.5 and later:
+/* .IP "\fBdetect_8bit_encoding_header (yes)\fR"
+/*	Automatically detect 8BITMIME body content by looking at
+/*	Content-Transfer-Encoding: message headers; historically, this
+/*	behavior was hard-coded to be "always on".
 /* AUTOMATIC BCC RECIPIENT CONTROLS
 /* .ad
 /* .fi
@@ -140,10 +272,10 @@
 /*	Available in Postfix version 2.1 and later:
 /* .IP "\fBsender_bcc_maps (empty)\fR"
 /*	Optional BCC (blind carbon-copy) address lookup tables, indexed
-/*	by sender address.
+/*	by envelope sender address.
 /* .IP "\fBrecipient_bcc_maps (empty)\fR"
 /*	Optional BCC (blind carbon-copy) address lookup tables, indexed by
-/*	recipient address.
+/*	envelope recipient address.
 /* ADDRESS TRANSFORMATION CONTROLS
 /* .ad
 /* .fi
@@ -167,7 +299,7 @@
 /*	off in email addresses.
 /* .IP "\fBmasquerade_exceptions (empty)\fR"
 /*	Optional list of user names that are not subjected to address
-/*	masquerading, even when their address matches $masquerade_domains.
+/*	masquerading, even when their addresses match $masquerade_domains.
 /* .IP "\fBpropagate_unmatched_extensions (canonical, virtual)\fR"
 /*	What address lookup tables copy an address extension from the lookup
 /*	key to the lookup result.
@@ -181,8 +313,11 @@
 /* .PP
 /*	Available in Postfix version 2.0 and later:
 /* .IP "\fBvirtual_alias_maps ($virtual_maps)\fR"
-/*	Optional lookup tables that alias specific mail addresses or domains
-/*	to other local or remote address.
+/*	Optional lookup tables that are often searched with a full email
+/*	address (including domain) and that apply to all recipients: \fBlocal\fR(8),
+/*	virtual, and remote; this is unlike alias_maps that are only searched
+/*	with an email address localpart (no domain) and that apply
+/*	only to \fBlocal\fR(8) recipients.
 /* .PP
 /*	Available in Postfix version 2.2 and later:
 /* .IP "\fBcanonical_classes (envelope_sender, envelope_recipient, header_sender, header_recipient)\fR"
@@ -194,9 +329,10 @@
 /*	What addresses are subject to sender_canonical_maps address
 /*	mapping.
 /* .IP "\fBremote_header_rewrite_domain (empty)\fR"
-/*	Don't rewrite message headers from remote clients at all when
-/*	this parameter is empty; otherwise, rewrite message headers and
-/*	append the specified domain name to incomplete addresses.
+/*	Rewrite or add message headers in mail from remote clients if
+/*	the remote_header_rewrite_domain parameter value is non-empty,
+/*	updating incomplete addresses with the domain specified in the
+/*	remote_header_rewrite_domain parameter, and adding missing headers.
 /* RESOURCE AND RATE CONTROLS
 /* .ad
 /* .fi
@@ -233,6 +369,26 @@
 /*	from each original recipient.
 /* .IP "\fBvirtual_alias_recursion_limit (1000)\fR"
 /*	The maximal nesting depth of virtual alias expansion.
+/* .PP
+/*	Available in Postfix version 3.0 and later:
+/* .IP "\fBvirtual_alias_address_length_limit (1000)\fR"
+/*	The maximal length of an email address after virtual alias expansion.
+/* SMTPUTF8 CONTROLS
+/* .ad
+/* .fi
+/*	Preliminary SMTPUTF8 support is introduced with Postfix 3.0.
+/* .IP "\fBsmtputf8_enable (yes)\fR"
+/*	Enable preliminary SMTPUTF8 support for the protocols described
+/*	in RFC 6531, RFC 6532, and RFC 6533.
+/* .IP "\fBsmtputf8_autodetect_classes (sendmail, verify)\fR"
+/*	Detect that a message requires SMTPUTF8 support for the specified
+/*	mail origin classes.
+/* .PP
+/*	Available in Postfix version 3.2 and later:
+/* .IP "\fBenable_idna2003_compatibility (no)\fR"
+/*	Enable 'transitional' compatibility between IDNA2003 and IDNA2008,
+/*	when converting UTF-8 domain names to/from the ASCII form that is
+/*	used for DNS lookups.
 /* MISCELLANEOUS CONTROLS
 /* .ad
 /* .fi
@@ -244,19 +400,19 @@
 /*	request before it is terminated by a built-in watchdog timer.
 /* .IP "\fBdelay_logging_resolution_limit (2)\fR"
 /*	The maximal number of digits after the decimal point when logging
-/*	sub-second delay values.
+/*	delay values.
 /* .IP "\fBdelay_warning_time (0h)\fR"
-/*	The time after which the sender receives the message headers of
-/*	mail that is still queued.
+/*	The time after which the sender receives a copy of the message
+/*	headers of mail that is still queued.
 /* .IP "\fBipc_timeout (3600s)\fR"
 /*	The time limit for sending or receiving information over an internal
 /*	communication channel.
 /* .IP "\fBmax_idle (100s)\fR"
-/*	The maximum amount of time that an idle Postfix daemon process
-/*	waits for the next service request before exiting.
+/*	The maximum amount of time that an idle Postfix daemon process waits
+/*	for an incoming connection before terminating voluntarily.
 /* .IP "\fBmax_use (100)\fR"
-/*	The maximal number of connection requests before a Postfix daemon
-/*	process terminates.
+/*	The maximal number of incoming connections that a Postfix daemon
+/*	process will service before terminating voluntarily.
 /* .IP "\fBmyhostname (see 'postconf -d' output)\fR"
 /*	The internet hostname of this mail system.
 /* .IP "\fBmyorigin ($myhostname)\fR"
@@ -273,13 +429,30 @@
 /*	the sender.
 /* .IP "\fBsyslog_facility (mail)\fR"
 /*	The syslog facility of Postfix logging.
-/* .IP "\fBsyslog_name (postfix)\fR"
-/*	The mail system name that is prepended to the process name in syslog
-/*	records, so that "smtpd" becomes, for example, "postfix/smtpd".
+/* .IP "\fBsyslog_name (see 'postconf -d' output)\fR"
+/*	A prefix that is prepended to the process name in syslog
+/*	records, so that, for example, "smtpd" becomes "prefix/smtpd".
 /* .PP
 /*	Available in Postfix version 2.1 and later:
 /* .IP "\fBenable_original_recipient (yes)\fR"
-/*	Enable support for the X-Original-To message header.
+/*	Enable support for the original recipient address after an
+/*	address is rewritten to a different address (for example with
+/*	aliasing or with canonical mapping).
+/* .PP
+/*	Available in Postfix 3.3 and later:
+/* .IP "\fBservice_name (read-only)\fR"
+/*	The master.cf service name of a Postfix daemon process.
+/* .PP
+/*	Available in Postfix 3.5 and later:
+/* .IP "\fBinfo_log_address_format (external)\fR"
+/*	The email address form that will be used in non-debug logging
+/*	(info, warning, etc.).
+/* .PP
+/*	Available in Postfix 3.9 and later:
+/* .IP "\fBforce_mime_input_conversion (no)\fR"
+/*	Convert body content that claims to be 8-bit into quoted-printable,
+/*	before header_checks, body_checks, Milters, and before after-queue
+/*	content filters.
 /* FILES
 /*	/etc/postfix/canonical*, canonical mapping table
 /*	/etc/postfix/virtual*, virtual mapping table
@@ -293,6 +466,7 @@
 /*	postconf(5), configuration parameters
 /*	master(5), generic daemon options
 /*	master(8), process manager
+/*	postlogd(8), Postfix logging
 /*	syslogd(8), system logging
 /* README FILES
 /* .ad
@@ -302,6 +476,7 @@
 /* .na
 /* .nf
 /*	ADDRESS_REWRITING_README Postfix address manipulation
+/*	CONTENT_INSPECTION_README content inspection
 /* LICENSE
 /* .ad
 /* .fi
@@ -311,6 +486,11 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
 /*--*/
 
 /* System library. */
@@ -334,6 +514,7 @@
 #include <mail_params.h>
 #include <record.h>
 #include <rec_type.h>
+#include <mail_version.h>
 
 /* Single-threaded server skeleton. */
 
@@ -362,7 +543,7 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
     /*
      * Open a queue file and initialize state.
      */
-    state = cleanup_open();
+    state = cleanup_open(src);
 
     /*
      * Send the queue id to the client. Read client processing options. If we
@@ -370,10 +551,11 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
      * about the whole operation.
      */
     attr_print(src, ATTR_FLAG_NONE,
-	       ATTR_TYPE_STR, MAIL_ATTR_QUEUEID, state->queue_id,
+	       SEND_ATTR_STR(MAIL_ATTR_PROTO, MAIL_ATTR_PROTO_CLEANUP),
+	       SEND_ATTR_STR(MAIL_ATTR_QUEUEID, state->queue_id),
 	       ATTR_TYPE_END);
     if (attr_scan(src, ATTR_FLAG_STRICT,
-		  ATTR_TYPE_NUM, MAIL_ATTR_FLAGS, &flags,
+		  RECV_ATTR_INT(MAIL_ATTR_FLAGS, &flags),
 		  ATTR_TYPE_END) != 1) {
 	state->errs |= CLEANUP_STAT_BAD;
 	flags = 0;
@@ -388,7 +570,13 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
      * extracted from message headers.
      */
     while (CLEANUP_OUT_OK(state)) {
-	if ((type = rec_get(src, buf, 0)) < 0) {
+	if ((type = rec_get_raw(src, buf, 0, REC_FLAG_NONE)) < 0) {
+	    state->errs |= CLEANUP_STAT_BAD;
+	    break;
+	}
+	if (REC_GET_HIDDEN_TYPE(type)) {
+	    msg_warn("%s: record type %d not allowed - discarding this message",
+		     state->queue_id, type);
 	    state->errs |= CLEANUP_STAT_BAD;
 	    break;
 	}
@@ -403,8 +591,15 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
      */
     if (CLEANUP_OUT_OK(state) == 0 && type > 0) {
 	while (type != REC_TYPE_END
-	       && (type = rec_get(src, buf, 0)) > 0)
-	     /* void */ ;
+	       && (type = rec_get_raw(src, buf, 0, REC_FLAG_NONE)) > 0) {
+	    if (type == REC_TYPE_MILT_COUNT) {
+		int     milter_count = atoi(vstring_str(buf));
+
+		/* Avoid deadlock. */
+		if (milter_count >= 0)
+		    cleanup_milter_receive(state, milter_count);
+	    }
+	}
     }
 
     /*
@@ -419,9 +614,11 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
      */
     status = cleanup_flush(state);		/* in case state is modified */
     attr_print(src, ATTR_FLAG_NONE,
-	       ATTR_TYPE_NUM, MAIL_ATTR_STATUS, status,
-	       ATTR_TYPE_STR, MAIL_ATTR_WHY, state->reason ?
-	       state->reason : "",
+	       SEND_ATTR_INT(MAIL_ATTR_STATUS, status),
+	       SEND_ATTR_STR(MAIL_ATTR_WHY,
+			     (state->flags & CLEANUP_FLAG_SMTP_REPLY)
+			     && state->smtp_reply ? state->smtp_reply :
+			     state->reason ? state->reason : ""),
 	       ATTR_TYPE_END);
     cleanup_free(state);
 
@@ -443,11 +640,18 @@ static void pre_accept(char *unused_name, char **unused_argv)
     }
 }
 
+MAIL_VERSION_STAMP_DECLARE;
+
 /* main - the main program */
 
 int     main(int argc, char **argv)
 {
 
+    /*
+     * Fingerprint executables and core dumps.
+     */
+    MAIL_VERSION_STAMP_ALLOCATE;
+
     /*
      * Clean up an incomplete queue file in case of a fatal run-time error,
      * or after receiving SIGTERM from the master at shutdown time.
@@ -459,14 +663,14 @@ int     main(int argc, char **argv)
      * Pass control to the single-threaded service skeleton.
      */
     single_server_main(argc, argv, cleanup_service,
-		       MAIL_SERVER_INT_TABLE, cleanup_int_table,
-		       MAIL_SERVER_BOOL_TABLE, cleanup_bool_table,
-		       MAIL_SERVER_STR_TABLE, cleanup_str_table,
-		       MAIL_SERVER_TIME_TABLE, cleanup_time_table,
-		       MAIL_SERVER_PRE_INIT, cleanup_pre_jail,
-		       MAIL_SERVER_POST_INIT, cleanup_post_jail,
-		       MAIL_SERVER_PRE_ACCEPT, pre_accept,
-		       MAIL_SERVER_IN_FLOW_DELAY,
-		       MAIL_SERVER_UNLIMITED,
+		       CA_MAIL_SERVER_INT_TABLE(cleanup_int_table),
+		       CA_MAIL_SERVER_BOOL_TABLE(cleanup_bool_table),
+		       CA_MAIL_SERVER_STR_TABLE(cleanup_str_table),
+		       CA_MAIL_SERVER_TIME_TABLE(cleanup_time_table),
+		       CA_MAIL_SERVER_PRE_INIT(cleanup_pre_jail),
+		       CA_MAIL_SERVER_POST_INIT(cleanup_post_jail),
+		       CA_MAIL_SERVER_PRE_ACCEPT(pre_accept),
+		       CA_MAIL_SERVER_IN_FLOW_DELAY,
+		       CA_MAIL_SERVER_UNLIMITED,
 		       0);
 }
